#ifndef CMDAREAD_H
/** \file

   Definitions to handle data in GSI's "AREAD" format.

   Copyright  2000-2009 M.Kraemer, GSI, D-64291 Darmstadt

   $Log: cmdaread.h,v $
   Revision 1.1.2.1  2005/11/22 17:16:47  kraemer
   sync'ed with old CMD

   Revision 1.1  2000/04/03 13:10:24  kraemer
   *** empty log message ***


*/
#define CMDAREAD_H

#include "cmderr.h"

                                      /* global flags */
#define CMDAREAD_DEBUG  0x80000000    /* general check flag */
#define CMDAREAD_QUIET  0x10000000    /* suppress error messages */
#define CMDAREAD_NOMSG  CMDAREAD_QUIET

#define CMDAREADWRITE_GD  0x00010000  /* function action flag */

#define CMDAREAD_MAXDIM 4

struct SCMDAREADWDW {
   long lvLim[CMDAREAD_MAXDIM][4];  
};
                                      /* structure to store data from file */
struct SCMDAREAD {
   unsigned long ulFlag;              /**< hold general flag bits */
   unsigned long ulSpace;
   double dvTime[2];                  /**< measuring time, [0/1] = real/live */ 
   char *pcAnl, *pcEvt;               /**< analyzer & event name */
   char *pcFile;                      /**< file name */    
   char *pcDate;                      /**< acq start date */    
   char *pcTime;                      /**< acq start time */    

   char **pvcCom;                     /**< [nCom] comments */ 
   int  nCom;

   int  nCal;
   double *pdCal;                     /**< [nCal] x-calibration coefficients */     

   unsigned long ulType;              /**< spectrum data type */
   int iSizeOf;
   long lDim;                         /**< dimension */
   unsigned long ulData;              /**< # of data items */
   int nWdw;
   struct SCMDAREADWDW *psWdw;        /**< [nWdw] window limits */ 
   long   lvLim[CMDAREAD_MAXDIM][4]; 
   double dvLim[CMDAREAD_MAXDIM][4]; 
   void *pv;                          /**< [ulData] */
   short *pi;  
   long  *pl;
   float *pr;   
   double *pd;                  
   struct SCMDERR sCE;
};
                                      /* flag bits above */
#define CMDAREAD_SPCREFCOPY 0x00000001
                                      /* spectrum data type */ 
enum {
   CMDAREAD_TYPESHORT=2, CMDAREAD_TYPELONG=4, CMDAREAD_TYPEFLOAT=24, CMDAREAD_TYPEDOUBLE=28,
};

/*

   Functions
   ---------
   
   cmdAReadFree      ( aread, flag )
                     o release resources
                   3 x to test
                     . honour refcopy flag
                     o calls:
                       o cmdAReadInit()
                            
   cmdAReadInit      ( aread, flag )
                     o initialize
                   2 x to test
                     o calls:
                         
   cmdAReadInspect   ( aread, flag )
                     o inspect   
                   1 x to test
                     o calls:
                         
   cmdAReadRead      ( aread, file, flag )
                     o read file in AREAD format
                   4 x to test
                     o determine data type from data format, if no type given
                     o multiple keywords on 1 line 
                     o allow/test free formatted data as well
                     o calls:
                       o cmdAReadInit     
                       o cmdAReadFree     

   cmdAReadWrite     ( aread, file, flag )
                     o write data in AREAD/gd format
                   5 o to test
                     o ar-format extension: 
                       o calibration
                       o windows/conditions
                     o gd:
                       o 2-dim
                       o binsize 
                       o anl, date, time
                       o calibration
                       . windows
                     o calls:
                       o cmdAReadInit     
                       o cmdAReadFree     

*/
                                      /* function prototypes */
int cmdAReadFree   ( struct SCMDAREAD *, unsigned long );
int cmdAReadInit   ( struct SCMDAREAD *, unsigned long );
int cmdAReadInspect( struct SCMDAREAD *, unsigned long );
int cmdAReadRead   ( struct SCMDAREAD *, const char *, unsigned long );
int cmdAReadWrite  ( struct SCMDAREAD *, const char *, unsigned long );

struct SCMDAREAD *_cmdAReadTest( unsigned long );

                                      /* error handling */
int cmdAReadError( struct SCMDERR *, unsigned long );

enum { ERRCMDAREAD_SYS=1,
       ERRCMDAREAD_BADADDR,
       ERRCMDAREAD_BADSYNTAX,
       ERRCMDAREAD_BADNUMBER, 
       ERRCMDAREAD_BADDIM,
       ERRCMDAREAD_BADDIMINDEX,
       ERRCMDAREAD_BADTYPE, 
       ERRCMDAREAD_BADCHANNEL, 
       ERRCMDAREAD_BADBINSIZE, 
       ERRCMDAREAD_BADNDATA, 
       ERRCMDAREAD_DATAINCONSISTENT, 
};

#endif

