#ifndef CMDSYS_H
#define CMDSYS_H
/** \file
   Common definitions for various low-level user interface (command) stuff.
*/
/*   
   
   Copyright  2002-2009 M.Kraemer, D-64291 Darmstadt, Seegartenstr. 32, 
                    all rights reserved

   $Log: cmdsys.h,v $
   Revision 1.1.2.2  2005/03/16 01:01:44  kraemer
   *** empty log message ***

   Revision 1.1.2.1  2005/03/15 14:57:24  kraemer
   *** empty log message ***


*/

#include <limits.h>
#include <ctype.h>

#include "u.h"
#include "cmderr.h"

#if defined(_AIX) || defined(__hpux) || defined(__mips) || defined(sun) || \
    defined(__linux__) || defined(__osf__) || defined(__APPLE_CC__) || \
    defined(ultrix) || defined(SUNOS4) 
#define CMDUNIX
#endif    

#if defined(CMDUNIX)

#include <fcntl.h>

#define CMDCURRENTDIR "./"

#elif defined(__EMX__)

#define CMDCURRENTDIR "./"
#define CMD_WMCLOSE (WM_USER+1)

#elif defined(MVS) || defined(mvs)

#elif defined(AMIGA)

#define CMDCURRENTDIR ""

#else
#error You loose, you should not use this computer
#endif

                                      /* masks */
#define CMD_MASKGLOBAL 0xFF000000     /* global (e.g. debug) */
#define CMD_MASKFUNC   0x00FF0000     /* function specific flags */
#define CMD_MASKQUAL   0x0000FFFF     /* for additional qualifiers */

                                      /* global action flags */
#define CMD_DEBUG      0x80000000     /* general check flag */
#define CMD_QUIET      0x40000000     /* suppress error messages */
#define CMD_NOMSG      CMD_QUIET

                                      /** \struct SCMDLOCK
                                          \brief A lock.
                                      */ 
struct SCMDLOCK {
#if defined(AMIGA)
   struct SignalSemaphore sSigSem;    /**< Exec semaphore */
#endif
#if defined(__EMX__)
   HMTX hmtx;                         /**< Dos Mutex */
#endif
#if defined(CMDUNIX)
   struct flock sLL;                  /**< Unix file lock specs */   
   struct flock sUL;                  /**< Unix file unlock specs */
   int iFile;                         /**< lock file handle */
   char cFile[L_tmpnam+1];            /**< lock file name */
#endif
};
                                      /* a lock macro */
#if defined(AMIGA)
#define CMDLOCK( psCL )\
   ObtainSemaphore( &((psCL) -> sSigSem) );
#elif defined(__EMX__)
#define CMDLOCK( psCL )\
   DosRequestMutexSem( (psCL) -> hmtx, SEM_INDEFINITE_WAIT );
#elif defined(CMDUNIX)
#define CMDLOCK( psCL )\
   {\
   fcntl( (psCL)->iFile, F_SETLKW /*blocks*/, &((psCL)->sLL) );\
   }
#else
#error You loose, you should not use this computer    
#endif

                                      /* an unlock macro */
#if defined(AMIGA)
#define CMDUNLOCK( psCL )\
   ReleaseSemaphore(  &((psCL) -> sSigSem)  );
#elif defined(__EMX__)
#define CMDUNLOCK( psCL )\
   DosReleaseMutexSem( (psCL) -> hmtx );
#elif defined(CMDUNIX)
#define CMDUNLOCK( psCL )\
   {\
   fcntl( (psCL)->iFile, F_SETLKW /*blocks*/, &((psCL)->sUL) );\
   }
#else
#error You loose, you should not use this computer       
#endif

                                      /* initialize macro */
#if defined(AMIGA)
#define CMDLOCKINIT( psCL )\
   memset( &((psCL) -> sSigSem), 0, sizeof((psCL) -> sSigSem) );
#elif defined(__EMX__)
#define CMDLOCKINIT( psCL )\
   (psCL) -> hmtx = NULLHANDLE;
#elif defined(CMDUNIX)
#define CMDLOCKINIT( psCL )\
   {\
   memset( (psCL), 0, sizeof(*(psCL)) );\
   (psCL)->iFile = -1;\
   }
#else
#error You loose, you should not use this computer       
#endif

                                      /* create a lock */
#if defined(AMIGA)
#define CMDLOCKCREATE( psCL, err )\
   InitSemaphore(  &((psCL) -> sSigSem)  );
#elif defined(__EMX__)
#define CMDLOCKCREATE( psCL, err )\
   DosCreateMutexSem( NULL, &( (psCL) -> hmtx ), DC_SEM_SHARED, FALSE );\
   DosOpenMutexSem( NULL, &( (psCL) -> hmtx ) );
#elif defined(CMDUNIX)
#define CMDLOCKCREATE( psCL, err )\
   (psCL) -> sLL.l_type = F_WRLCK;\
   (psCL) -> sLL.l_whence = SEEK_SET;\
   (psCL) -> sLL.l_start = 0;\
   (psCL) -> sLL.l_len = 0;\
   (psCL) -> sUL.l_type = F_UNLCK;\
   tmpnam( (psCL) -> cFile);\
   (psCL) -> iFile = open( (psCL) -> cFile,\
                           O_RDWR | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR );\
   if ( (psCL) -> iFile < 0 ) {\
      err;\
   }  
#else
#error You loose, you should not use this computer       
#endif

                                      /* delete a lock */
#if defined(AMIGA)
#define CMDLOCKFREE( psCL ) ;
#elif defined(__EMX__)
#define CMDLOCKFREE( psCL )\
   DosCloseMutexSem( (psCL) -> hmtx );
#elif defined(CMDUNIX)
#define CMDLOCKFREE( psCL )\
   if ( (psCL)->iFile >= 0 ) {\
      close( (psCL)->iFile ); remove( (psCL)->cFile );\
   }
#else
#error You loose, you should not use this computer       
#endif
                                      /* inspect a lock */
#if defined(AMIGA)
#define CMDLOCKINSPECT( psCL ) ;
#elif defined(__EMX__)
#define CMDLOCKINSPECT( psCL )\
   fprintf( stderr, "          mtx=%X\n", (psCL) -> hmtx );
#elif defined(CMDUNIX)
#define CMDLOCKINSPECT( psCL )\
   fprintf( stderr, "     lockfile=%d %s\n", (psCL)->iFile, (psCL)->cFile );
#else
#error You loose, you should not use this computer       
#endif

                                      /* valid char in commands */
#define ISCMDCMDCHAR(c)  ( isalpha(c) || isdigit(c) || ( ((c) & 127) == 0x5f) || ( (c) == '$' )  )
                                      /* valid first char in commands */
#define ISCMDCMDCHARF(c) ( isalpha(c)               || ( ((c) & 127) == 0x5f) || ( (c) == '$' )  )

                                      /* valid char in parameters */
#define ISCMDPARCHAR(c)  ( isalpha(c) || isdigit(c) || ( ((c) & 127) == 0x5f) || ( (c) == '$' )  )


/*
   --- error handling
*/

int _cmdError( struct SCMDERR *, unsigned long );

enum { ERRCMD_SYS=1,
       ERRCMD_BADADDR,
       ERRCMD_BADCMDINIT,  
       ERRCMD_SYNTAX,
       ERRCMD_INIT, 
       ERRCMD_BALANCE, 
       ERRCMD_CONV,
       ERRCMD_CMDITEM, 
       ERRCMD_PARITEM, 
       ERRCMD_NODEABBREV, 
       ERRCMD_NODEADD,
       ERRCMD_ALIAS, 
       ERRCMD_CMDNOTFOUND,
       ERRCMD_BADFCT, 
       ERRCMD_MAXPAR, 
       ERRCMD_CMDNOPAR,
       ERRCMD_DEF2VAL, 
       ERRCMD_VALUE,
       ERRCMD_DUPLICATE, 
       ERRCMD_VAL2DEF, 
       ERRCMD_BADQUOTES,
       ERRCMD_PARTOOMANY, 
       ERRCMD_PARNOTFOUND, 
       ERRCMD_PARREQTOOFEW,
       ERRCMD_PARNOTOPT, 
       ERRCMD_SHELL,
       ERRCMD_EXECLEVEL, 
       ERRCMD_EXECNOFILE, 
       ERRCMD_EXECSYNTAX,
       ERRCMD_EXECNOPROMPT, 
       ERRCMD_EXECJUMPSELECT,
       ERRCMD_EXECMULTIWHEN, 
       ERRCMD_EXECSELECTBALANCE,
       ERRCMD_EXECBADLEVEL,
       ERRCMD_EXECBADSTOP,
       ERRCMD_CREATEMSGPORTFAILED,
       ERRCMD_BADFILEHANDLE,
       ERRCMD_ALLOCDOSOBJECTFAILED,
       ERRCMD_HELPFULLNOACTION,
       ERRCMD_BADCMDNAME,
       ERRCMD_BADPARNAME,
       ERRCMD_CMDDEFINEFAILED,
       ERRCMD_CMDBADFCT,
       ERRCMD_CMDPAREXISTS,
       ERRCMD_CMDPARNOTYET,
       ERRCMD_CMDNOTYET,       
       ERRCMD_IPARSETUPFAILED,  
       ERRCMD_IPARBADNAMES,
       ERRCMD_IPARDCLFAILED,
       ERRCMD_IPARLISTFAILED,
       ERRCMD_IPARFROMSTRFAILED,
       ERRCMD_IPARLISTADDFAILED,
       ERRCMD_IPARNOTSCALAR,
       ERRCMD_MATHTREEFROMSTRFAILED,
       ERRCMD_MATHTREEEVALUATEFAILED,          
       ERRCMD_MATHTREEDATAALLOCFAILED,
       ERRCMD_WINREGISTERCLASSFAILED,
       ERRCMD_HWNDCLIENTFAILED, 
       ERRCMD_OPENGADTOOLSFAILED,
       ERRCMD_OPENASLFAILED,
       ERRCMD_OPENINTUITIONFAILED,
};
#endif
